library(shiny)
library(tidyverse)

# Create a Shiny application using the sidebarLayout and the iris dataset, plotting the species vs the Sepal.Length.
# Remember you can use a browser actionButton to help with debugging 
# In the UI code - actionButton("browser", "browser")
# In the server code - observeEvent(input$browser, browser())

# 
# 1 a. In the sidebarPanel, include a textInput to allow the user to name the plot.
#   b. In the main panel, show a box plot of the iris data, with the Species along the x axis and Sepal.Length on the y axis. 
#   c. ggplot objects can be saved and then called when needed. Move the plotting code out of the render function into a reactive expression and then call the reactive expression from within the render function. 
#   d. Add actionButtons in the sidebarPanel to allow the user to show either a boxplot, violin plot or bar plot. Use geom_col() for the barplot. 
#     Create a reactive value using reactiveVal() and call it plotting_object. 
#     Pair each of the actionButtons with an observeEvent() function and within these, update the plotting_object reactive value. 
#     Call plotting_object() from the renderPlot function to display the plot.
# 
#      
# 2. Add a checkboxInput that when clicked, shows the individual points on the plots.
# 
# Think about the different options you have for where to put this code (within existing or new reactive expressions, observeEvents or render functions) and what would happen and why if you ran the code from different places.


ui <- fluidPage(
  
  titlePanel("Exercise 3"),
  
  sidebarLayout(
    sidebarPanel(
      textInput(inputId = "plot_name", label = "Enter plot name"),
      br(),
      actionButton(inputId = "boxplot", label = "boxplot"),
      actionButton(inputId = "violinplot", label = "violinplot"),
      actionButton(inputId = "barplot", label = "barplot"),
      br(),
      checkboxInput(inputId = "add_points", label = "show data points")
    ),
    mainPanel(
      plotOutput(outputId = "iris_plot"),
    )
  ),
  #actionButton("browser", "browser")
)

server <- function(input, output, session) {
  
  plotting_object <- reactiveVal()
  
  plot_base <- reactive({
    iris %>%
      ggplot(aes(x = Species, y = Sepal.Length, fill = Species)) +
      ggtitle(input$plot_name)
      
  })
  
  observeEvent(input$boxplot, {
    
    p <- plot_base() +
      geom_boxplot()
    
    plotting_object(p)
  })
  
  observeEvent(input$barplot, {
    
    p <- plot_base() +
      geom_col()
    
    plotting_object(p)
  })
  
  observeEvent(input$violinplot, {
    
    p <- plot_base() +
      geom_violin()
    
    plotting_object(p)
  })
  
  output$iris_plot <- renderPlot({
    
    p <- plotting_object()

    if(input$add_points) {
      p + geom_jitter(width = 0.2, height = 0)
    } else {
      p
    }
  })
  
  observeEvent(input$browser, {
    browser()
  })
  
}

shinyApp(ui, server)

